@@ -2,16 +2,16 @@ package Net::DNS::Resolver::Unbound;
 
 use strict;
 use warnings;
-
-our $VERSION;
-$VERSION = '1.18_03';
-
-use Carp;
 use Net::DNS;
-
 use base qw(Net::DNS::Resolver DynaLoader);
-eval { __PACKAGE__->bootstrap($VERSION) };
-warn "\n\n$@\n" if $@;
+
+BEGIN {
+	our $VERSION;
+	$VERSION = '1.19_01';
+	eval { __PACKAGE__->bootstrap($VERSION) };
+}
+
+use constant UB_SEND => Net::DNS::Resolver::Unbound::Context->can('ub_send');
 
 
 =head1 NAME
@@ -112,7 +112,7 @@ sub send {
 	$self->_reset_errorstring;
 
 	my $query = $self->_make_query_packet(@_);
-	my ($q)	  = $query->question;
+	my ($q) = $query->question;
 	my $reply = $self->{ub_ctx}->ub_resolve( $q->name, $q->{qtype}, $q->{qclass} );
 	return $self->_decode_result( $reply, $query->header->id );
 }
@@ -142,10 +142,8 @@ sub bgread {
 
 	$self->{ub_ctx}->ub_wait if &bgbusy;
 
-	my $async_id = $handle->async_id;
 	$self->errorstring( $handle->err );
-	my $result = $handle->result;
-	return $self->_decode_result( $result, $async_id );
+	return $self->_decode_result( $handle->result, $handle->query_id );
 }
 
 
@@ -404,17 +402,15 @@ sub print {
 sub _decode_result {
 	my ( $self, $result, $query_id ) = @_;
 
-	my $packet;
-	if ($result) {
-		$self->errorstring('INSECURE') unless $result->secure;
-		$self->errorstring( $result->why_bogus ) if $result->bogus;
+	return unless $result;
 
-		my $buffer = $result->answer_packet || return;
-		substr( $buffer, 0, 2 ) = pack 'n', $query_id;	# emulate Net::DNS resolver->send(packet)
-		$packet = Net::DNS::Packet->decode( \$buffer, $self->debug );
-		$self->errorstring($@);
-	}
+	$self->errorstring('INSECURE') unless $result->secure;
+	$self->errorstring( $result->why_bogus ) if $result->bogus;
 
+	my $buffer = $result->answer_packet || return;
+	my $packet = Net::DNS::Packet->decode( \$buffer, $self->debug );
+	$self->errorstring($@);
+	$packet->header->id($query_id) if !UB_SEND || $query_id;
 	return $packet;
 }
 
